# uploadCopybook.pl - Upload a Copybook file
#
# Authors:  Jane Huang
# Last updated:  3/22/07
#			Xiao Lin Zhang, 7/06/09
#      Xiao Lin Zhang 28/6/10
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is spawned from the Copybook_main.pl script for each Copybook
# file that was selected to be uploaded.  
#
# The calling syntax for this script is:
# 	Usage: uploadCopybook.pl [-1] -d directory_path -f filename
# Where;
# 	-1 indicates that the uploading list is one file which causes
#          the log output to be displayed to stdout as well as the log file
# 	-i identifies the directory where the profileUSER are located
# 	-d identifies the directory where the script and logs exist
# 	-f identifies the file to be uploaded
#
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 
use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Getopt::Std;
use bccpbmod;

my $debug      = $bccpbmod::debug;     # for verbose debugging output
my $debug      = 1;     # for verbose debugging output

my $rccVerbosity = "";
$rccVerbosity = '-V -V -V' if $debug;

my $sysout       = $bccpbmod::sysout;
$sysout = 'RCCEXT=RCCOUT,DISP=(NEW,CATLG,DELETE),SPACE=(32000,(30,30))' if $debug;

my $jcl;	    # JCL filename
my $ncpycnt = 7;    # number of CICS NewCopy steps
my $lcnt    = 0;    # general loop counter

# #############################################################################
# process input arguments
#
#
my $usage = "uploadCopybook [-1] -i Path_profileUSER -d dir -f file";


my $compile_1 = 0; # Compilation from main consisted of one file
my $dir_file  = ''; # directory where the file resides
my $filename  = ''; # filename - full or relative path

if (!getopts('1i:d:f:') || $opt_h) {
    print "$usage \n";
    exit(0);
}


$compile_1 = $opt_1;
$dir_file  = $opt_d;
$filename  = $opt_f;
$Path_profileUSER= $opt_i;


# *******************************************************************************
# * Required customization:                                                     *
# * Judge OS type to determine the delimiter embedded in path, you can modify it*
# * according to you site's requirement or you can hardcode delimiter here.     *
# ******************************************************************************* 
my $delimiter = '';
my $pcid = '';
my $current_dir = '';

$pcid=$ENV{USERNAME} || $ENV{USER} || $ENV{LOGNAME};

my $OS = $^O;
if ($OS =~/.*Win.*/){	#For Windows operating system
	$delimiter = "\\";
	$current_dir = `cd`;
}elsif ($OS =~/.*n.*x/ or $OS eq "hpux" or $OS eq "solaris"){	# For Unix & Linux operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}elsif ($OS eq "aix"){	#For AIX operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}
print "Upload script: current dir =  $current_dir \n" if $debug;

print "Upload script: One file = $compile_1 \n" if $debug;
print "Upload script: filename = $filename \n" if $debug;
print "Upload script: dir_file = $dir_file \n" if $debug;


# #############################################################################
# Determine the directory context for the execution
#

($scriptdir,$lifecycle,$EDfile,$VIEWfile,$listdir,$logdir,$perlexe) = bccpbmod::getprofileUSER($Path_profileUSER,$delimiter);

# #############################################################################
# Get the environment variables from the common module
# These variables establish the distributed and mainframe environment
# and must be modified for each installation.  The values for these 
# variables are held in projectED file.
# These varibles are used to point to the Z/OS infrastructure/environment/mapped 
# Life cycle that the project will take. This includes SYSLIB concatenation to be
# used for each environment ie: TEST, QA, PROD. What target loadlibs are to be used
# DB2 subsystem, CICS region, IMS region etc. They are declared at the project level
# the are queried and used here.
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.                                                                      *
# *******************************************************************************
my ($env, $db2sys, $os390_hostname, $os390_port, $stage,
$copylib1, $copylib2, $copylib3, $copylib4,
$syslib1, $syslib2, $syslib3,
#$syslib4, $steplib1, $steplib2,			 # optional
#$hostpds_prefix, $hostpds_affix, $cbllib,	         # optional 
$loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) = bccpbmod::setEnvironmentVar($EDfile,$lifecycle,$delimiter);

# print "Env: $bccpbmod::env \n" if $debug;
# print "db2sys: $bccpbmod::db2sys \n" if $debug;


# #############################################################################
# Translate the distributed username to a mainframe ID
#

my $mfid = bccpbmod::getUserID($scriptdir,$pcid,$delimiter);  
bccpbmod::my_die('Unknown User.') if ($mfid eq "");

# *******************************************************************************
# * Optional customization:                                                     *
# * The following code can be used to generate the PDS if the type attribute    *
# * ZCCENV is set with the $hostpds_prefix and $hostpds_affix.                  *
# *******************************************************************************

  my $name = $mfid;
#  my $affix ="";
#  my $prefix ="";
#  if (($hostpds_prefix ne '') || ($hostpds_affix ne '')) {
#     if ($hostpds_prefix ne '') {
#        $prefix = $hostpds_prefix . ".";
#     }
#     if ($hostpds_affix ne '') {
#        $affix = "." . $hostpds_affix ;
#     }
#     $name = $prefix . $mfid . $affix ;
#   } 
################################################################################
print "PCID: $pcid, MFID: $mfid, NAME: $name\n" if $debug;

# *******************************************************************************
# * Required customization:                                                     *
# * define location for jcl which will be generated during build request        *
# ******************************************************************************* 


# #############################################################################
# Cleanup JCL files from the previous execution
#
#bccpbmod::cleanup($jcldir,$delimiter);

# #############################################################################
# determine the member name from the clearcase filename. The member name is
# just the filename without the front path and extension.
#
my $mem_ext = basename($filename);
my $dot      = rindex( $mem_ext, '.' );
my $mem      = substr( $mem_ext, 0, $dot );
my $ext      = substr( $mem_ext, $dot+1 );

my $jcldir = $dir_file.$delimiter.$mfid.$delimiter.$mem;

printf( "Upload script: processing file (%s)\n", $filename ) if $debug;
printf( "Upload script: mem (%s) ext (%s)\n", $mem, $ext ) if $debug;

# *******************************************************************************
# * Required customization:                                                     *
# * The following code has to be modified to meet your site's requirement.      *
# *******************************************************************************
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
                 
bccpbmod::my_die('Allocation error.')                     if ( s1alloc() != 0 );   
bccpbmod::my_die('Upload Copybook error.')                if ( uploadcopy() != 0 ); 

# Otherwise, exit successfully
exit 0;     # test only - comment out by jane


##############################################################
# The following subroutines contain each of the JCL decks
# that execute specific mainframe tasks
#
# Each subroutine performs the following functions:
#    1) build JCL file name
#    2) write specific JCL to the file
#    3) execute the JCL
#    4) report status
#
##############################################################
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< s1alloc >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub s1alloc {
    $jcl = 's1alloc_' . $mem .'.jcl';
    $jcl = $jcldir . $delimiter . $jcl;
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    bccpbmod::writeFile(
        $jcl, $compile_1, "
//* ------------------------------------------------------------------
//S1ALLOC  EXEC PGM=IEFBR14
//*        ----
//*        ---- IEFBR14 - Upload and Allocate Temporary Files
//*        ----
//TEMP     DD  DSN=$name.XCOPYLIB,
//             DISP=(MOD,KEEP,DELETE),
//             UNIT=SYSDA,SPACE=(CYL,(5,5,5)),
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=3200)
//* ------------------------------------------------------------------
"
    );

    my $cmd =
"\"$bccpbmod::rccbuild\" -h $os390_hostname\@$os390_port -b S1ALLOC -ft $jcl  -n 0 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

    print "Step: s1alloc cmd: $cmd\n " if $debug;
    my $status = 0;

    $status = bccpbmod::rcc_system($compile_1, $cmd);
    printf( "s1alloc Status was (%d)\n", $status ) if $debug;


    return $status;
}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< uploadcopy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub uploadcopy {
    $jcl = 'upload_' . $mem .'.jcl';
    $jcl = $jcldir . $delimiter . $jcl;
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    bccpbmod::writeFile(
        $jcl, $compile_1, "
//* ------------------------------------------------------------------
//UPLOAD   EXEC PGM=IEFBR14
//*        ----
//*        ---- IEFBR14 - Upload and Allocate Copybook Files
//*        ----
//UPLD1    DD  DSN=$name.XCOPYLIB,RCCEXT=CPY,
//             DISP=OLD
//* ------------------------------------------------------------------
"
    );

    my $cmd =
"\"$bccpbmod::rccbuild\" -h $os390_hostname\@$os390_port -b UPLOAD -ft $jcl -i ..$delimiter..$delimiter$filename -n 0 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

    print "Step: s1alloc cmd: $cmd\n " if $debug;
    my $status = 0;

    $status = bccpbmod::rcc_system($compile_1, $cmd);
    printf( "upload Status was (%d)\n", $status ) if $debug;


    return $status;
}

